\name{coords.aniso}

\alias{coords.aniso}

%- Also NEED an `\alias' for EACH other topic documented here.

\title{(Geometric) anisotropy correction}

\description{
  Transform or back-transform a set of coordinates according to
  the parameters defining the geometric anisotropy.
}

\usage{
coords.aniso(coords, aniso.pars, reverse = FALSE)
}

\arguments{
  \item{coords}{an \eqn{n \times 2}{n x 2} matrix with the coordinates
    to be transformed.  }
  \item{aniso.pars}{a vector with two elements: the \emph{anisotropy
      angle} and the \emph{anisotropy ratio} necessarily in this order.
    See section DETAILS below for the description of these elemets.  }
  \item{reverse}{Defaults to \code{FALSE}. If \code{TRUE} the reverse
    transformation is performed.  }
}

\details{
  The parameters defining the geometric anisotropy are:
  \enumerate{
    \item{\emph{Anisotropy angle}} is defined here as the azimuth angle of the
    direction with greater spatial continuity, i.e. the angle between the
    y-axis and the direction of the maximum range.
    
    \item{\emph{Anisotropy ratio}} is defined here as the ratio between the ranges
    of the directions with greater and smaller continuity, i.e. the ratio
    between maximum and minimum ranges. It is therefore always
    grater than one.
  }
    
  If \code{reverse = FALSE} (the default) the
  coordinates are transformed  from the \emph{anisotropic space} to the \emph{isotropic
    space}. The transformation is done by multiplying the original
  coordinates by a rotation matrix \eqn{R}
  and a
  shrinking matrix \eqn{T}, as follows:
  \deqn{X_m = X \%*\% R \%*\% T}
  where \eqn{X_m} is a matrix with the modified coordinates (isotropic
  space) , \eqn{X} is a matrix with original coordinates (anisotropic
  space), \eqn{R} rotates coordinates according to the anisotropy angle
  and \eqn{T} shrinks the coordinates according to the anisotropy ratio.
  
  If \code{reverse = TRUE}, the back-transformation is performed and the
  coordinates are therefore transformed  from the \emph{isotropic space} to the \emph{anisotropic
    space} by computing:
  \deqn{X = X_m (R T)^{-1}}{X = X_m \%*\% solve( R \%*\% T ).}

}


\value{
  An \eqn{n \times 2}{n x 2} matrix with the transformed coordinates.
}

\references{
  Further information about \code{geoR/geoS} can be found at:\cr
  \url{http://www.maths.lancs.ac.uk/~ribeiro/geoR.html}
}

\author{
  Paulo Justiniano Ribeiro Jr. \email{paulojus@est.ufpr.br} \cr
  Peter J. Diggle \email{p.diggle@lancaster.ac.uk}.
}

%\note{ ~~further notes~~ }

 
%\seealso{ ~~objects to SEE ALSO as \code{\link{~~fun~~}}, ~~~ }

\examples{
par.mf <- par()$mfrow
par(mfrow=c(3,2))
par(mar=c(2.5,0,0,0))
par(mgp=c(2,.5,0))
par(pty="s")
## Defining a set of coordinates
coords <- expand.grid(seq(-1, 1, l=3), seq(-1, 1, l=5))
plot(c(-1.5, 1.5), c(-1.5, 1.5), xlab="", ylab="", type="n")
points(coords)
## Transforming coordinates according to some anisotropy parameters
coordsA <- coords.aniso(coords, aniso.pars=c(0, 2))
plot(c(-1.5, 1.5), c(-1.5, 1.5), xlab="", ylab="", type="n")
points(coordsA)
##
coordsB <- coords.aniso(coords, aniso.pars=c(pi/2, 2))
plot(c(-1.5, 1.5), c(-1.5, 1.5), xlab="", ylab="", type="n")
points(coordsB)
##
coordsC <- coords.aniso(coords, aniso.pars=c(pi/4, 2))
plot(c(-1.5, 1.5), c(-1.5, 1.5), xlab="", ylab="", type="n")
points(coordsC)
##
coordsD <- coords.aniso(coords, aniso.pars=c(3*pi/4, 2))
plot(c(-1.5, 1.5), c(-1.5, 1.5), xlab="", ylab="", type="n")
points(coordsD)
##
coordsE <- coords.aniso(coords, aniso.pars=c(0, 5))
plot(c(-1.5, 1.5), c(-1.5, 1.5), xlab="", ylab="", type="n")
points(coordsE)
##
par(mfrow=par.mf)
}

\keyword{spatial}
